@@ -38,6 +38,7 @@ class PaymentSubTypes(str, Enum):
     webatm = "webatm"
     cvs = "cvs"
     unknown = "unknown"
+    default = "default"  # ecpay payment all open
 
 
 class LogisticTypes(str, Enum):
@@ -50,6 +51,7 @@ class LogisticTypes(str, Enum):
 class HomeSubtypes(str, Enum):
     ECAN = "ECAN"
     TCAT = "TCAT"
+    SF = "SF"
 
 
 class InvoiceTypes(str, Enum):
@@ -91,8 +93,8 @@ class CartConfig(BaseCartConfig):
     receiver_tel: str = ""
     receiver_tel_ext: str = ""
     payment_subtype: PaymentSubTypes = PaymentSubTypes.credit
-    logistics_type: LogisticTypes = LogisticTypes.CVS
-    logistics_subtype: str = CVSLogisticSubTypes.FAMIC2C
+    logistics_type: LogisticTypes = LogisticTypes.HOME
+    logistics_subtype: str = HomeSubtypes.TCAT
     logistics_cvs_store_id: Optional[str]
 
     invoice_type: InvoiceTypes = InvoiceTypes.B2C
