@@ -67,7 +67,6 @@ class SdkConfigCls:
     sf_secret: str = ""
     sf_card_no: str = ""
 
-    default_payment_provider: str = "neweb"
     payment_stepfn_arn: str = ""
     logistics_stepfn_arn: str = ""
 
@@ -206,9 +205,6 @@ def sdk_auto_config(sdk_client: str = "") -> None:
         SdkConfig.conversion_token = get_env_or_raise(
             "CONVERSION_TOKEN", raise_exception=False
         )
-        SdkConfig.default_payment_provider = get_env_or_raise(
-            "DEFAULT_PAYMENT_PROVIDER", False
-        )
         SdkConfig.payment_stepfn_arn = get_env_or_raise("PAYMENT_STEPFN_ARN", False)
         SdkConfig.logistics_stepfn_arn = get_env_or_raise("LOGISTICS_STEPFN_ARN", False)
         SdkConfig.neweb_merchant_id = get_env_or_raise("NEWEB_MERCHANT_ID", False)
