@@ -1,4 +1,3 @@
-import time
 from typing import Any, Dict, List, Tuple, Union
 
 import boto3
@@ -11,7 +10,11 @@ from requests import Response as ResponseType
 from fastel.collections import get_site_config
 from fastel.config import SdkConfig
 from fastel.exceptions import APIException
-from fastel.logistics.common.models import LogisticsDataModel, LogisticsStatus
+from fastel.logistics.common.models import (
+    LogisticsDataModel,
+    LogisticsStatus,
+    LogisticTypes,
+)
 from fastel.logistics.ecpay.gateway import EcpayLogistics
 from fastel.logistics.ecpay.models import (
     CvsMapSubTypeOptions,
@@ -24,8 +27,7 @@ from fastel.logistics.ecpay.models import (
 from fastel.logistics.sf.gateway import SFLogistics
 from fastel.logistics.sf.models import SFLogisticsModel
 from fastel.payment.common.models.order import Order, ProductItem
-from fastel.utils import requests
-from fastel.utils.requests import parse_formdata
+from fastel.utils import requests, timestamp_postfix
 
 
 class EcpayLogisticsCreate:
@@ -122,11 +124,21 @@ class EcpayLogisticsCreate:
     @staticmethod
     def parse_gateway_resp(gateway_resp: Dict[str, Any]) -> LogisticsDataModel:
         if gateway_resp.get("error", {}):
-            raise APIException(
-                status_code=500, error="ECPAY_ERROR", detail=gateway_resp["error"]
+            error = "error"
+            error_msg = gateway_resp["error"]
+            return LogisticsDataModel(
+                provider="ecpay",
+                order_id=error,
+                logistics_id=error,
+                logistics_type=LogisticTypes.UNKNOWN,
+                logistics_subtype="",
+                logistics_status=LogisticsStatus.error,
+                logistics_message=error_msg,
+                logistics_detail=gateway_resp,
             )
 
         return LogisticsDataModel(
+            provider="ecpay",
             order_id=gateway_resp["MerchantTradeNo"].split("_")[0],
             logistics_id=gateway_resp["AllPayLogisticsID"],
             logistics_type=gateway_resp["LogisticsType"],
@@ -155,7 +167,7 @@ class EcpayLogisticsCallback:
 
     async def parse_request(self, request: Request) -> Dict[str, Any]:
         data = await request.form()
-        return parse_formdata(data)
+        return requests.parse_formdata(data)
 
     def validate(self, data: Dict[str, Any]) -> EcpayLogisticsCallbackModel:
         return EcpayLogisticsCallbackModel.validate(data)
@@ -252,6 +264,7 @@ class EcpayLogisticsCallback:
                 logistics_status = LogisticsStatus.exception
 
         return LogisticsDataModel(
+            provider="ecpay",
             order_id=result.MerchantTradeNo.split("_")[0],
             logistics_id=result.AllPayLogisticsID,
             logistics_type=result.LogisticsType,
@@ -293,7 +306,11 @@ class EcpayGetShippingNote:
         soup = BeautifulSoup(html)
         img_tag = soup.find("img")
         if not img_tag:
-            return ""
+            form = soup.find("form")
+            if not form:
+                return ""
+            else:
+                return str(form.get("action", ""))
         else:
             return str(img_tag.get("src", ""))
 
@@ -301,20 +318,20 @@ class EcpayGetShippingNote:
         return LogisticsDataModel.validate(data)
 
     def parse_payload(self, result: LogisticsDataModel) -> EcpayPrintOrderInfo:
-        if result.logsitics_subtype == "UNIMARTC2C":
+        if result.logistics_subtype == "UNIMARTC2C":
             payload = {
                 "LogisticsSubType": "UniMartC2C",
                 "AllPayLogisticsID": result.logistics_id,
                 "CVSPaymentNo": result.logistics_payment_no,
                 "CVSValidationNo": result.logistics_validate_no,
             }
-        elif result.logsitics_subtype in ["FAMIC2C", "OKMARTC2C", "HILIFEC2C"]:
+        elif result.logistics_subtype in ["FAMIC2C", "OKMARTC2C", "HILIFEC2C"]:
             payload = {
-                "LogisticsSubType": result.logsitics_subtype,
+                "LogisticsSubType": result.logistics_subtype,
                 "AllPayLogisticsID": result.logistics_id,
                 "CVSPaymentNo": result.logistics_payment_no,
             }
-        elif result.logsitics_subtype in [
+        elif result.logistics_subtype in [
             "FAMI",
             "UNIMART",
             "UNIMARTFREEZE",
@@ -323,7 +340,7 @@ class EcpayGetShippingNote:
             "ECAN",
         ]:
             payload = {
-                "LogisticsSubType": result.logsitics_subtype,
+                "LogisticsSubType": result.logistics_subtype,
                 "AllPayLogisticsID": result.logistics_id,
             }
 
@@ -461,10 +478,11 @@ class SFLogisticsCreate:
 
         else:
             logistics_id = ""
-            logistics_status = LogisticsStatus.exception
+            logistics_status = LogisticsStatus.error
             logistics_message = gateway_resp.get("msg", "")
 
         return LogisticsDataModel(
+            provider="sf",
             order_id=resp_data.get("customerOrderNo", ""),
             logistics_id=logistics_id,
             logistics_type="UNKNOW",
@@ -509,6 +527,7 @@ class SFLogisticsStatus:
             logistics_message = gateway_resp.get("msg", "")
 
         return LogisticsDataModel(
+            provider="sf",
             order_id="",
             logistics_id=logistics_id,
             logistics_status=logistics_status,
@@ -559,15 +578,15 @@ def get_logistics_create_method(
 ) -> Union[EcpayLogisticsCreate, SFLogisticsCreate]:
     if provider == "ecpay":
         return EcpayLogisticsCreate(
-            callback_url=f"{SdkConfig.api_host}/{prefix_path}/callback",
+            callback_url=f"{SdkConfig.api_host}/{prefix_path}/{provider}/callback",
         )
     elif provider == "sf":
         return SFLogisticsCreate()
     raise ValueError("unrecognize logistics provider")
 
 
-def get_callback_method() -> EcpayLogisticsCallback:
-    if SdkConfig.default_payment_provider == "ecpay":
+def get_callback_method(provider: str = "ecpay") -> EcpayLogisticsCallback:
+    if provider == "ecpay":
         return EcpayLogisticsCallback()
     raise ValueError("unrecognize payment provider")
 
@@ -587,16 +606,15 @@ def get_shipping_note_method(provider: str = "ecpay") -> EcpayGetShippingNote:
 stepfn = boto3.client("stepfunctions", "ap-northeast-1")
 
 
-async def process_callback(request: Request) -> Response:
-    tm = int(time.time())
-    callback_method = get_callback_method()
+async def process_callback(provider: str, request: Request) -> Response:
+    callback_method = get_callback_method(provider)
     data = await callback_method.parse_request(request)
     callback_result = callback_method(data)
     # to avoid step function name duplicate error
-    stepfn_name = f"{callback_result.order_id}_{callback_result.logistics_status}_{tm}"
+    stepfn_name = f"{callback_result.order_id}_{callback_result.logistics_status}"
     stepfn.start_execution(
         stateMachineArn=SdkConfig.logistics_stepfn_arn,
-        name=stepfn_name,
+        name=timestamp_postfix(stepfn_name),
         input=callback_result.json(),
     )
     return callback_method.respond(order_id=callback_result.order_id)
