@@ -8,7 +8,7 @@ from fastapi.responses import HTMLResponse, RedirectResponse
 
 from fastel.cart import Cart
 from fastel.cart.datastructures import InvoiceTypes, PaymentSubTypes
-from fastel.collections import get_collection
+from fastel.collections import get_collection, get_site_config
 from fastel.config import SdkConfig
 from fastel.payment.common.models.callback import CallbackModel
 from fastel.payment.common.models.invoice import InvoiceIssueResp, InvoiceStatus
@@ -41,7 +41,7 @@ from fastel.payment.neweb.models.callback import (
 from fastel.payment.neweb.models.checkout import CheckoutModel
 from fastel.payment.neweb.models.invoice import IssueModel, IssueResp
 from fastel.payment.utils import PaymentStatus, auto_order_number
-from fastel.utils import TW
+from fastel.utils import TW, timestamp_postfix
 from fastel.utils.requests import parse_formdata
 
 
@@ -247,23 +247,49 @@ class NewebInvoiceIssue:
 
         if gateway_resp.Result:
             invoice_number = gateway_resp.Result.InvoiceNumber
+            invoice_random_no = gateway_resp.Result.RandomNum
         else:
             invoice_number = ""
+            invoice_random_no = ""
 
         return InvoiceIssueResp(
             status=status,
             invoice_message=gateway_resp.Message,
             invoice_number=invoice_number,
             invoice_detail=gateway_resp.dict(),
+            invoice_random_no=invoice_random_no,
         )
 
     def validate(self, order: Dict[str, Any]) -> Order:
         return Order.validate(order)
 
     def parse_order(self, order: Order) -> IssueModel:
+        # items = order.items + order.extra_items + order.discount_items
         item_name = "|".join(list(map(lambda item: item.name, order.items)))
         item_count = "|".join(list(map(lambda item: str(item.config.qty), order.items)))
         item_unit = "|".join(list(map(lambda item: "個", order.items)))
+        _amount = ""
+        _price = ""
+        for extra_item in order.extra_items:
+            item_name += "|" + extra_item.get("name", "")
+            item_count += "|1"
+            item_unit += "|個"
+            if order.invoice_type == InvoiceTypes.B2B:
+                _amount += "|" + str(round(extra_item.get("amount", 0) / 1.05, 2))
+                _price += "|" + str(round(extra_item.get("amount", 0) / 1.05, 2))
+            else:
+                _amount += "|" + str(extra_item.get("amount", 0))
+                _price += "|" + str(extra_item.get("amount", 0))
+        for discount_item in order.discount_items:
+            item_name += "|" + discount_item.get("name", "")
+            item_count += "|1"
+            item_unit += "|個"
+            if order.invoice_type == InvoiceTypes.B2B:
+                _amount += "|-" + str(round(discount_item.get("sale_amount", 0), 2))
+                _price += "|-" + str(round(discount_item.get("sale_amount", 0), 2))
+            else:
+                _amount += "|-" + str(discount_item.get("amount", 0))
+                _price += "|-" + str(discount_item.get("amount", 0))
         if order.invoice_type == InvoiceTypes.B2B:
             item_amount = "|".join(
                 list(map(lambda item: str(round(item.sales_amount)), order.items))
@@ -276,7 +302,8 @@ class NewebInvoiceIssue:
                 list(map(lambda item: str(item.amount), order.items))
             )
             item_price = "|".join(list(map(lambda item: str(item.price), order.items)))
-
+        item_amount += _amount
+        item_price += _price
         payload: Dict[str, Any] = {"BuyerName": order.buyer_name}
 
         if order.invoice_type == InvoiceTypes.B2B:
@@ -381,11 +408,13 @@ class EcpayCheckout:
             type_map["ChoosePayment"] = "BARCODE"
             type_map["PaymentInfoURL"] = self.payment_info_url
             type_map["ClientRedirectURL"] = self.order_result_url
-        elif cart.payment_subtype == PaymentSubTypes.unknown:
+        elif cart.payment_subtype == PaymentSubTypes.default:
             type_map["ChoosePayment"] = "ALL"
             type_map["PaymentInfoURL"] = self.payment_info_url
             type_map["ClientRedirectURL"] = self.order_result_url
             type_map["OrderResultURL"] = self.order_result_url
+        else:
+            raise ValueError("invalid Payment Subtypes")
 
         name = "#".join([item.name for item in cart.items.items])
         item_name = self._replace_limit_name(name=name, limit=400)
@@ -588,6 +617,7 @@ class EcpayInvoiceIssue:
             invoice_message=resp_data.get("RtnMsg", ""),
             invoice_number=invoice_number,
             invoice_detail=gateway_resp,
+            invoice_random_no=resp_data.get("RandomNumber", ""),
         )
 
 
@@ -718,19 +748,20 @@ class LinePayCallback:
 def get_checkout_method(
     prefix_path: str = "payment",
 ) -> Union[NewebpayCheckout, EcpayCheckout, LinePayCheckout]:
-    if SdkConfig.default_payment_provider == "neweb":
+    default_payment_provider = get_site_config("default_payment_provider", "ecpay")
+    if default_payment_provider == "neweb":
         return NewebpayCheckout(
             callback_url=f"{SdkConfig.api_host}/{prefix_path}/callback",
             number_url=f"{SdkConfig.api_host}/{prefix_path}/number",
             redirect_url=f"{SdkConfig.api_host}/{prefix_path}/return",
         )
-    elif SdkConfig.default_payment_provider == "ecpay":
+    elif default_payment_provider == "ecpay":
         return EcpayCheckout(
             callback_url=f"{SdkConfig.api_host}/{prefix_path}/callback",
             number_url=f"{SdkConfig.api_host}/{prefix_path}/number",
             redirect_url=f"{SdkConfig.api_host}/{prefix_path}/return",
         )
-    elif SdkConfig.default_payment_provider == "linepay":
+    elif default_payment_provider == "linepay":
         return LinePayCheckout(
             callback_url=f"{SdkConfig.api_host}/{prefix_path}/callback",
             error_url=f"{SdkConfig.web_host}/misc?title=結帳發生錯誤&url={SdkConfig.web_host}&btn_text=返回",
@@ -740,27 +771,30 @@ def get_checkout_method(
 
 
 def get_callback_method() -> Union[NewebpayCallback, EcpayCallback, LinePayCallback]:
-    if SdkConfig.default_payment_provider == "neweb":
+    default_payment_provider = get_site_config("default_payment_provider", "ecpay")
+    if default_payment_provider == "neweb":
         return NewebpayCallback()
-    elif SdkConfig.default_payment_provider == "ecpay":
+    elif default_payment_provider == "ecpay":
         return EcpayCallback()
-    elif SdkConfig.default_payment_provider == "linepay":
+    elif default_payment_provider == "linepay":
         return LinePayCallback()
     raise ValueError("unrecognize payment provider")
 
 
 def get_getnum_method() -> Union[NewebpayGetNumber, EcpayGetNumber]:
-    if SdkConfig.default_payment_provider == "neweb":
+    default_payment_provider = get_site_config("default_payment_provider", "ecpay")
+    if default_payment_provider == "neweb":
         return NewebpayGetNumber()
-    elif SdkConfig.default_payment_provider == "ecpay":
+    elif default_payment_provider == "ecpay":
         return EcpayGetNumber()
     raise ValueError("unrecognize payment provider")
 
 
 def get_invoice_issue_method() -> Union[NewebInvoiceIssue, EcpayInvoiceIssue]:
-    if SdkConfig.default_payment_provider == "neweb":
+    default_payment_provider = get_site_config("default_payment_provider", "ecpay")
+    if default_payment_provider == "neweb":
         return NewebInvoiceIssue()
-    elif SdkConfig.default_payment_provider == "ecpay":
+    elif default_payment_provider == "ecpay":
         return EcpayInvoiceIssue()
     raise ValueError("unrecognize payment provider")
 
@@ -772,9 +806,12 @@ async def process_callback(request: Request) -> Response:
     callback_method = get_callback_method()
     data = await callback_method.parse_request(request)
     callback_result = callback_method(data)
+    stepfn_name = (
+        f"callback_{callback_result.order_id}_{callback_result.payment_status}"
+    )
     stepfn.start_execution(
         stateMachineArn=SdkConfig.payment_stepfn_arn,
-        name=f"callback_{callback_result.order_id}",
+        name=timestamp_postfix(stepfn_name),
         input=callback_result.json(),
     )
 
@@ -795,9 +832,10 @@ async def process_getnum(request: Request) -> Response:
     getnum_method = get_getnum_method()
     data = await getnum_method.parse_request(request)
     getnum_result = getnum_method(data)
+    stepfn_name = f"getnum_{getnum_result.order_id}_{getnum_result.payment_status}"
     stepfn.start_execution(
         stateMachineArn=SdkConfig.payment_stepfn_arn,
-        name=f"getnum_{getnum_result.order_id}",
+        name=timestamp_postfix(stepfn_name),
         input=getnum_result.json(),
     )
 
